#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "dns.h"

int udp_port = 161;

/*
Sending 43 bytes to 192.168.1.101:161
0000: 30 29 02 01  00 04 06 70  75 62 6C 69  63 A0 1C 02    0).....public..
0016: 04 75 42 B2  08 02 01 00  02 01 00 30  0E 30 0C 06    .uB.......0.0..
0032: 08 2B 06 01  02 01 01 01  00 05 00                    .+.........
*/

/* Aaaaarrrgggghhhh.... */
char query[43] = {
  0x30, 0x29, 0x02, 0x01, 0x00, 0x04,
  6, 'p', 'u', 'b', 'l', 'i', 'c',

  0xA0,
  0x1C,

  0x02, 0x04, 0xDE, 0xAD, 0xCA, 0xFE,

  0x02, 0x01, 0x00, 0x02, 0x01, 0x00, 0x30, 0x0E, 0x30, 0x0C, 0x06,
  0x08, 0x2B, 0x06, 0x01, 0x02, 0x01, 0x01, 0x01, 0x00, 0x05, 0x00
};


int
format_packet (void *buffer, int maxlen, struct in_addr addr)
{
  memcpy (buffer, query, sizeof (query));

  return sizeof (query);
}

/*
Received 307 bytes from 192.168.1.101:161
0000: 30 82 01 2F**02 01 00**04 06 'public'           A2    0../.....public
0016: 82 01 20**02 04 DEADCAFE*  02 01 00**02 01 00** 30    .. ..uB.......0
0032: 82 01 10 30  82 01 0C 06  08 2B 06 01  02 01 01 01    ...0.....+......
0048: 00 04 81 FF  43 69 73 63  6F 20 49 6E  74 65 72 6E    ...Cisco Intern
0064: 65 74 77 6F  72 6B 20 4F  70 65 72 61  74 69 6E 67    etwork Operating
*/

char *forms[2] = { "0:Primitive", "1:Constructed" };
char *classes[4] =
  { "0:Universal", "1:Application", "2:Context", "3:Private" };

int savenextstring;

char oid[8] = { 0x2B, 0x06, 0x01, 0x02, 0x01, 0x01, 0x01, 0x00 };
char name[0x40];

int
savename (char *buffer, int len)
{
  int i, nlen;


  for (i = 0, nlen = 0; i < len && nlen < 0x3F; i++)
    if (buffer[i] >= 0x20)
      name[nlen++] = buffer[i];

  name[nlen] = 0;
}


int
asn1_decode (unsigned char *buffer, int len)
{
  int class, frm, id;
  int datalen;
  int i, j, k;


  while (len > 2)
    {

      id = *buffer & 0x1F;
      frm = (*buffer >> 5) & 0x01;
      class = (*buffer >> 6) & 0x03;
      buffer++;
      len--;

      if (id == 0x1F)
	{
	  /*printf ("ASN-ID-EXT\n"); */
	  for (id = 0; (*buffer & 0x80) && (len > 0); len--, buffer++)
	    id = (id << 7) | (*buffer & 0x7F);
	}

      if (*buffer & 0x80)
	{
	  j = *buffer;
	  buffer++;
	  len--;
	  datalen = 0;
	  for (i = 0; i < (j & 0x7F); i++)
	    {
	      datalen *= 0x100;
	      datalen |= (*buffer);
	      buffer++;
	      len--;
	    }
	}
      else
	{
	  datalen = *buffer;
	  len--;
	  buffer++;
	}

/*
      printf ("class %s, frm %s, id %d, len %d\n", classes[class], forms[frm],
	      id, datalen);
*/
      if (datalen > len)
	{
	  /*printf ("error! datalen > len\n"); */
	  break;
	}

      switch (frm)
	{
	case 0:		/* PRIMITIVE */
	  switch (class)
	    {
	    case 0:		/* UNIVERSAL */
	      switch (id)
		{
		case 6:
		  if (datalen == 8 && !memcmp (buffer, oid, 8))
		    savenextstring = 1;
		  break;
		case 4:	/* STRING */
		  if (savenextstring)
		    {
		      savename (buffer, datalen);
		    }
		  savenextstring = 0;
		  break;
		}		/* switch id -- PRIMITIVE:UNIVERSAL */
	    }			/* switch class -- PRIMITIVE */
	  break;
	case 1:		/* CONSTRUCTED */
	  switch (class)
	    {
	    case 0:		/* UNIVERSAL */
	      switch (id)
		{
		case 16:
		  /*printf ("<<<\n"); */
		  asn1_decode (buffer, datalen);
		  /*printf (">>>\n"); */
		  break;
		}		/* switch id -- CONSTRUCTED:UNIVERSAL */
	      break;
	    case 2:		/* CONTEXT */
	      switch (id)
		{
		case 2:	/* GetResponse */
		  /*printf ("<<<\n"); */
		  asn1_decode (buffer, datalen);
		  /*printf (">>>\n"); */
		  break;
		}		/* switch id -- CONSTRUCTED:CONTEXT */

	    }			/* switch class */
	  break;
	}			/* switch frm */

      len -= datalen;
      buffer += datalen;
    }
}

int
received_packet (void *buffer, int len, struct in_addr addr)
{
  savenextstring = 0;

  name[0] = 0;
  asn1_decode ((unsigned char *) buffer, len);

  if (name[0] == 0)
    printf ("%s <unknown>\n", inet_ntoa (addr));
  else
    printf ("%s %s\n", inet_ntoa (addr), name);


  return 1;
}
